clear all;
close all;

N = 4096;

sigma_x2 = 25;

x = sqrt(sigma_x2)*randn(1,10^6);

B = [1];
A = [1 -2.6934 2.4748 -0.6627 0.1519 -0.5061 0.2801];

y = filter(B, A, x);

window = window(@hamming, N);
[P_nonparam, W] = pwelch(y, window, length(window)/2, N, 'twosided');

plot(W, 10*log10(P_nonparam));
xlabel('Frequenz [rad]');
ylabel('Leistung/rad [dB]');
title('Nicht-parametrischer PSD Schtzer nach Welch');
axis([0, pi, -20, 60]);
grid on;

% Verwende Analysegleichung lt. MATLAB:
P_xx = sigma_x2/(2*pi);
P_analytic = P_xx * squaredspectrum(B, A, N);

figure;
plot(W, 10*log10(P_analytic));
xlabel('Frequenz [rad]');
ylabel('Leistung/rad [dB]');
title('Analytische Lsung der PSD');
axis([0, pi, -20, 60]);
grid on;

P_param = zeros(N, 10);

times = zeros(1, 10);

for m=1:10
    tic;
    HBurg = spectrum.burg(m);
    HSpec = psd(HBurg, y, 'NFFT', N, 'SpectrumType','twosided');
    P_param(:,m) = HSpec.Data;
    times(m) = toc;
end

figure;
plot(W, 10*log10(P_param));
xlabel('Frequenz [rad]');
ylabel('Leistung/rad [dB]');
title('Parametrischer PSD Schtzer nach Burg');
axis([0, pi, -20, 60]);
grid on;
legend('m=1', 'm=2', 'm=3', 'm=4', 'm=5', ...
    'm=6', 'm=7', 'm=8', 'm=9', 'm=10');


figure;
plot(W, [P_param(:,1) P_param(:,2) P_analytic]);
xlabel('Frequenz [rad]');
ylabel('Leistung/rad');
title('Vergleich zwischen m=1, m=2 und der analytischen Lsung');
axis([0, pi, 0, 7*10^5]);
grid on;
legend('m=1', 'm=2', 'analytisch');




P = repmat(P_analytic, 1, 10);
e = sum(abs(P_param - P))/(size(P,1));
mse = sum(abs(P_param - P).^2)/(size(P,1));
rmse = sqrt(mse);

mse_db = sum(abs(20*log10(P_param ./ P)))/size(P,1);

figure;
plot(e);
xlabel('Anzahl der Koeffizienzen');
ylabel('Absoluter Fehler');
title('Fehler pro Anzahl der Koeffizienten');
grid on;

figure;
plot(mse_db);
xlabel('Anzahl der Koeffizienzen');
ylabel('Quadratischer Fehler [dB]');
title('Fehler pro Anzahl der Koeffizienten in dB');
grid on;


figure;
plot(mse);
xlabel('Anzahl der Koeffizienzen');
ylabel('Quadratischer Fehler');
title('Fehler pro Anzahl der Koeffizienten');
grid on;


figure;
plot(rmse);
xlabel('Anzahl der Koeffizienzen');
ylabel('RMSE');
title('RMSE pro Anzahl der Koeffizienten');
grid on;

figure;
plot(times);
title('Laufzeit');
xlabel('Anzahl Koeffizienten');
ylabel('Zeit [s]');
grid on;



mse_nonparam = sum(abs(P_nonparam - P_analytic).^2)/size(P,1);
mse_nonparam_db = sum(abs(20*log10(P_nonparam ./ P_analytic)))/size(P,1);




























