clear all;
close all;

% Gewnschte Spezifikation
% -1dB Verstrkung im Durchlassbereich
K1 = -1;
% -35dB Verstrkung im Sperrbereich
K2 = -35;
% Abtastfrequenz: 50 Hz
Fs = 50;
% Nutzsignal: 1 Hz
F1 = 1;
% Strsignal: 5 Hz
F2 = 5;
% Fuer Bilineartransformation
T = 1;


W1 = F1*2*pi;
W2 = F2*2*pi;

w1 = W1/Fs;
w2 = W2/Fs;

% Prewarping
W1_wrapped = 2*tan(w1/2)/T;
W2_wrapped = 2*tan(w2/2)/T;

% Ordnung berechnen
n = log10(...
    (10^(-K1/10)-1)/(10^(-K2/10)-1)) / ...
    (2*log10(W1_wrapped/W2_wrapped));

n_unwrapped = log10(...
    (10^(-K1/10)-1)/(10^(-K2/10)-1)) / ...
    (2*log10(W1/W2));
N = ceil(n);

% Grenzfrequenz berechnen
Wc_unwrapped = W2/(10^(-K2/10)-1)^(1/(2*n_unwrapped));
Wc = W2_wrapped/(10^(-K2/10)-1)^(1/(2*n));
% oder alternativ:
% Wc_unwrapped = W1/(10^(-K1/10)-1)^(1/(2*n));
% Wc = W1_wrapped/(10^(-K1/10)-1)^(1/(2*n));


% kontinuierlicher Filter
W = linspace(0, 20*pi, 10000);
F = W/(2*pi);
Hc = 1./(1 + 2*(j*W/Wc_unwrapped) + ...
    2*(j*W/Wc_unwrapped).^2 + (j*W/Wc_unwrapped).^3);

figure;
plot(F, 20*log10(abs(Hc)));
grid on;
title('Kontinuierlicher Tiefpassfilter');
xlabel('Frequenz [Hz]');
ylabel('Verstrung [dB]');

% diskreter Filter nach Bilineartransformation
% Koeffizientenherleitung siehe Protokoll
B = [1, 3, 3, 1];
A = [ 1 + 4/(Wc * T) + 8/(Wc^2 * T^2) +  8/(Wc^3 * T^3) , ...
      3 + 4/(Wc * T) - 8/(Wc^2 * T^2) - 24/(Wc^3 * T^3) , ...
      3 - 4/(Wc * T) - 8/(Wc^2 * T^2) + 24/(Wc^3 * T^3) , ...
      1 - 4/(Wc * T) + 8/(Wc^2 * T^2) -  8/(Wc^3 * T^3) ];

w = linspace(0, pi, 1000);
f = w*(Fs/2)/pi;
z = exp(j*w);
H = (B * [z.^0 ; z.^-1 ; z.^-2 ; z.^-3]) ./ ...
    (A * [z.^0 ; z.^-1 ; z.^-2 ; z.^-3]);

figure;
plot(f, 20*log10(abs(H)));
grid on;
title('Zeitdiskreter Tiefpassfilter');
xlabel('Frequenz [Hz]');
ylabel('Verstrung [dB]');


% Test des Filters
t = 0:(1/Fs):10-1/Fs;
x = 5*sin(2*pi*F1*t) + 2*sin(2*pi*F2*t);
y = filter(B, A, x);

figure;
subplot(2, 1, 1);
plot(t, x);
grid on;
title('Einganssignal x[n]');
xlabel('Zeit [s]');
subplot(2, 1, 2);
plot(t, y);
grid on;
title('Ausgangssignal y[n]');
xlabel('Zeit [s]');

ww = linspace(0, 2*pi, length(y));
X = abs(fft(x))/length(x);
Y = abs(fft(y))/length(y);
figure;
subplot(1, 2, 1);
plot(ww, X);
grid on;
title('Eingangsspektrum X[k]');
xlabel('Zeit [s]');
subplot(1, 2, 2);
plot(ww, Y);
grid on;
title('Ausgangsspektrum Y[X]');
xlabel('Zeit [s]');













